from cryptography import x509
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.x509.oid import NameOID
from cryptography.exceptions import InvalidSignature
import datetime

def build_cert(key, signing_key, subject, issuer, issue_delta = 0, expiry_delta = 50, hash_alg = hashes.SHA256()):

    cert = x509.CertificateBuilder().subject_name(
        subject
    ).issuer_name(
        issuer
    ).public_key(
        key.public_key()
    ).serial_number(
        x509.random_serial_number()
    ).not_valid_before(
        datetime.datetime.utcnow() - datetime.timedelta(days=issue_delta)
    ).not_valid_after(
        datetime.datetime.utcnow() + datetime.timedelta(days=expiry_delta)
    # Sign our certificate with our private key
    ).sign(signing_key, hash_alg)
    return cert

def validate_cert_signature_rsa(cert, issuer_public_key):
    try:
        issuer_public_key.verify(
            cert.signature,
            cert.tbs_certificate_bytes,
            # Depends on the algorithm used to create the certificate
            padding.PKCS1v15(),
            cert.signature_hash_algorithm,
        )
    except InvalidSignature:
        return False
    return True

def read_cert_from_file(filepath):
    with open(filepath, "rb") as f:
        cert = x509.load_pem_x509_certificate(f.read())
    return cert